//+------------------------------------------------------------------+
//|                                        DT DB CODE CLAUDE 7.3.mq5 |
//|                                                              JFX |
//|                                                           JM-JFX |
//+------------------------------------------------------------------+

#property copyright "JFX"
#property link      "JM-JFX"
#property version   "7.3"
#property strict

#include <Trade/Trade.mqh>
#include <Math/Stat/Stat.mqh>

//--- Panel Inputs
input group "===== Trading Panel Configuration ====="
input bool EnableTradingPanel = true;  // Enable Trading Panel
input int PanelX = 50;                 // Panel X Position
input int PanelY = 50;                 // Panel Y Position
input int PanelWidth = 250;            // Panel Width
input int PanelHeight = 300;           // Panel Height

//--- Trading Direction Inputs
input group "===== Trading Direction ====="
input bool AllowBuy = true;            // Allow Buy Trades
input bool AllowSell = true;           // Allow Sell Trades
input double BuyLotPercentage = 1.0;   // Buy Lot Size Percentage
input double SellLotPercentage = 1.0;  // Sell Lot Size Percentage

//--- Trading Session Inputs
input group "===== Trading Sessions ====="
input bool EnableAsianSession = true;
input bool EnableLondonSession = true;
input bool EnableNYOpen = true;
input bool EnableNYClose = true;
input int AsianSessionStart = 0;    // Asian Session Start (GMT Hour)
input int AsianSessionEnd = 9;      // Asian Session End (GMT Hour)
input int LondonSessionStart = 8;   // London Session Start (GMT Hour)
input int LondonSessionEnd = 17;    // London Session End (GMT Hour)
input int NYOpenStart = 13;          // NY Open Start (GMT Hour)
input int NYOpenEnd = 20;            // NY Open End (GMT Hour)
input int NYCloseStart = 20;         // NY Close Start (GMT Hour)
input int NYCloseEnd = 22;           // NY Close End (GMT Hour)

//--- Multiple Symbol Inputs
input group "===== Symbol Configuration ====="
input string TradableSymbols = "EURUSD,GBPUSD,USDJPY,USDCHF,AUDUSD,NZDUSD"; // Tradable Symbols (Comma-Separated)

//--- Pattern and Timeframe Inputs
input group "===== Pattern Detection ====="
input ENUM_TIMEFRAMES PatternTimeframe = PERIOD_H1; // Pattern Detection Timeframe
input int LookbackBars = 1000;       // Bars to look back for pattern checks
input double DTDBThreshold = 0.0003; // DT/DB Price Similarity Threshold

//--- Risk Management Inputs
input group "===== Risk Management ====="
input double BaseLotSize = 0.01;     // Base Lot Size
input ENUM_RISK_CALCULATION RiskMethod = RISK_PERCENT; // Risk Calculation Method
input double RiskPercent = 1.0;      // Risk Percentage per Trade
input int ATRPeriod = 14;            // ATR Period for Stop Loss/Take Profit
input double ATRStopLossMultiplier = 1.5;  // ATR Multiplier for Stop Loss
input double ATRTakeProfitMultiplier = 2.0; // ATR Multiplier for Take Profit

//--- Breakeven and Trailing Stop Inputs
input group "===== Profit Management ====="
input bool EnableBreakeven = true;   // Enable Breakeven
input double BreakevenTriggerPercent = 50; // Breakeven Trigger Percentage
input double BreakevenBufferPercent = 0.5;  // Breakeven Buffer Percentage

input bool EnableTrailingStop = true; // Enable Trailing Stop
input double TrailingStopStartPercent = 30; // Start Trailing After Profit Percent
input double TrailingStopPercent = 20;  // Trailing Stop Percentage

//--- Advanced Partial Profit Management
input group "===== Partial Profit Management ====="
input bool EnablePartialProfits = true;  // Enable Partial Profit Taking
input double PartialProfit1Percent = 20; // First Partial Profit Level
input double PartialProfit2Percent = 40; // Second Partial Profit Level
input double PartialProfit3Percent = 60; // Third Partial Profit Level
input double PartialProfit4Percent = 80; // Fourth Partial Profit Level
input double PartialProfit5Percent = 100; // Final Profit Level

//---PARTIAL TP %
input int PartialProfit1Pips = 50;     // Pips for first partial profit
input int PartialProfit2Pips = 100;    // Pips for second partial profit
input int PartialProfit3Pips = 150;    // Pips for third partial profit
input int PartialProfit4Pips = 200;    // Pips for fourth partial profit
input int PartialProfit5Pips = 250;    // Pips for fifth partial profit

//---HFT
input ENUM_TIMEFRAMES HTF1 = PERIOD_M30; // Asian session start timeframe
input ENUM_TIMEFRAMES HTF2 = PERIOD_H1; // London session start timeframe
input ENUM_TIMEFRAMES HTF3 = PERIOD_H1; // New York open timeframe
input ENUM_TIMEFRAMES HTF4 = PERIOD_H4; // New York close timeframe


input int RefreshTimer = 100;          // Refresh timer in milliseconds
input bool UseATRStopLoss = true;      // Use ATR-based Stop Loss
input double ATROffset = 1.5;          // ATR offset multiplier for Stop Loss
input uint MagicNumber = 123456;       // Magic number for orders
input bool Show_Debug_Comments = true; // Display debug info on chart
input bool OneTradePerSignal = true;   // Only one trade per detected signal
input bool Enable_Buy = true;          // Enable buy trades
input bool Enable_Sell = false;        // Enable sell trades (extend for bearish patterns if needed)


//--- Advanced Pattern Inputs
input group "===== Advanced Pattern Detection ====="
input bool UseStrongEngulfing = true;
input bool UseRetestConfirmation = true;
input bool UsePiercingPattern = true;
input bool UseMorningStar = true;
input bool UseThreeInsideUp = true;
input bool UseThreeOutsideUp = true;
input bool UseHammerPattern = true;
input bool UseHangingManPattern = true;

//--- Visualization Inputs
input group "===== Visualization ====="
input color EngulfingColor = clrGreen;
input color PiercingColor = clrOrange;
input color MorningStarColor = clrDodgerBlue;
input color EveningStarColor = clrRed;
input color InsideUpColor = clrYellow;
input color InsideDownColor = clrGold;
input color OutsideUpColor = clrViolet;
input color OutsideDownColor = clrPurple;
input color StarSouthColor = clrAqua;
input color StarNorthColor = clrSkyBlue;
input color HangingManColor = clrTomato;
input color HammerColor = clrLime;
input string FontName = "Arial";
input int FontSize = 10;

//--- Global Variables
CTrade Trade;
MqlRates htfRates[];
string g_SymbolList[];
datetime g_LastTradeTime = 0;
int g_RefreshTimer = 50; // Refresh timer in milliseconds

// Performance Tracking
double g_DailyProfit = 0;
double g_WeeklyProfit = 0;
double g_MonthlyProfit = 0;
datetime g_LastDailyReset = 0;
datetime g_LastWeeklyReset = 0;
datetime g_LastMonthlyReset = 0;

// Profit Tracking Panel
int g_PanelLabelProfit;
int g_PanelLabelBuy;
int g_PanelLabelSell;

//--- Function Prototypes
bool IsValidTradingSession();
bool IsSymbolTradable(string symbol);
double CalculatePositionSize(string symbol, double riskAmount, bool isBuy);
void ManageBreakeven(ulong ticket);
void ManageTrailingStop(ulong ticket);
void ManagePartialProfits(ulong ticket);
void UpdateProfitTrackingPanel();
void ResetProfitTracking();

// Candlestick Pattern Functions (From Candle Pattern file)
bool IsStrongEngulfing(int index);
bool IsPiercingPattern(int index);
bool IsMorningStar(int index);
bool IsEveningStar(int index);
bool IsThreeInsideUp(int index);
bool IsThreeInsideDown(int index);
bool IsThreeOutsideUp(int index);
bool IsThreeOutsideDown(int index);
bool IsHangingMan(int index);
bool IsHammer(int index);
void DrawPatternLabel(int index, string patternName, color patternColor);

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
   // Split tradable symbols
   StringSplit(TradableSymbols, ',', g_SymbolList);
   
   // Initialize Profit Tracking Panel
   if(EnableTradingPanel)
   {
      // Create panel background
      ObjectCreate(0, "PanelBackground", OBJ_RECTANGLE_LABEL, 0, 0, 0);
      ObjectSetInteger(0, "PanelBackground", OBJPROP_XDISTANCE, PanelX);
      ObjectSetInteger(0, "PanelBackground", OBJPROP_YDISTANCE, PanelY);
      ObjectSetInteger(0, "PanelBackground", OBJPROP_XSIZE, PanelWidth);
      ObjectSetInteger(0, "PanelBackground", OBJPROP_YSIZE, PanelHeight);
      ObjectSetInteger(0, "PanelBackground", OBJPROP_BGCOLOR, C'240,240,240');
      ObjectSetInteger(0, "PanelBackground", OBJPROP_BORDER_TYPE, BORDER_SUNKEN);
      
      // Profit Label
      g_PanelLabelProfit = ObjectCreate(0, "ProfitLabel", OBJ_LABEL, 0, 0, 0);
      ObjectSetInteger(0, "ProfitLabel", OBJPROP_XDISTANCE, PanelX + 10);
      ObjectSetInteger(0, "ProfitLabel", OBJPROP_YDISTANCE, PanelY + 10);
      ObjectSetString(0, "ProfitLabel", OBJPROP_TEXT, "Total Profit: $0.00");
      ObjectSetInteger(0, "ProfitLabel", OBJPROP_COLOR, clrBlack);
      
      // Buy Button
      g_PanelLabelBuy = ObjectCreate(0, "BuyButton", OBJ_BUTTON, 0, 0, 0);
      ObjectSetInteger(0, "BuyButton", OBJPROP_XDISTANCE, PanelX + 10);
      ObjectSetInteger(0, "BuyButton", OBJPROP_YDISTANCE, PanelY + 50);
      ObjectSetInteger(0, "BuyButton", OBJPROP_XSIZE, PanelWidth - 20);
      ObjectSetString(0, "BuyButton", OBJPROP_TEXT, "Buy " + DoubleToString(BuyLotPercentage, 2) + "%");
      
      // Sell Button
      g_PanelLabelSell = ObjectCreate(0, "SellButton", OBJ_BUTTON, 0, 0, 0);
      ObjectSetInteger(0, "SellButton", OBJPROP_XDISTANCE, PanelX + 10);
      ObjectSetInteger(0, "SellButton", OBJPROP_YDISTANCE, PanelY + 90);
      ObjectSetInteger(0, "SellButton", OBJPROP_XSIZE, PanelWidth - 20);
      ObjectSetString(0, "SellButton", OBJPROP_TEXT, "Sell " + DoubleToString(SellLotPercentage, 2) + "%");
   }
   
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   // Clean up panel objects
   ObjectDelete(0, "PanelBackground");
   ObjectDelete(0, "ProfitLabel");
   ObjectDelete(0, "BuyButton");
   ObjectDelete(0, "SellButton");
   
   // Clear pattern labels
   ObjectsDeleteAll(0, 0, OBJ_TEXT);
   ObjectsDeleteAll(0, 0, OBJ_RECTANGLE);
}

//+------------------------------------------------------------------+
//| Reset Profit Tracking                                            |
//+------------------------------------------------------------------+
void ResetProfitTracking()
{
   MqlDateTime currentTime;
   TimeCurrent(currentTime);
   
   // Daily Reset
   if(g_LastDailyReset == 0 || currentTime.day != TimeDay(g_LastDailyReset))
   {
      g_DailyProfit = 0;
      g_LastDailyReset = TimeCurrent();
   }
   
   // Weekly Reset (Sunday)
   if(g_LastWeeklyReset == 0 || currentTime.day_of_week == 0)
   {
      g_WeeklyProfit = 0;
      g_LastWeeklyReset = TimeCurrent();
   }
   
   // Monthly Reset
   if(g_LastMonthlyReset == 0 || currentTime.mon != TimeMonth(g_LastMonthlyReset))
   {
      g_MonthlyProfit = 0;
      g_LastMonthlyReset = TimeCurrent();
   }
}

//+------------------------------------------------------------------+
//| Update Profit Tracking Panel                                     |
//+------------------------------------------------------------------+
void UpdateProfitTrackingPanel()
{
   if(!EnableTradingPanel) return;
   
   ResetProfitTracking();
   
   // Calculate total profit from all positions
   double totalProfit = 0;
   for(int i = 0; i < PositionsTotal(); i++)
   {
      ulong ticket = PositionGetTicket(i);
      if(ticket > 0)
      {
         totalProfit += PositionGetDouble(POSITION_PROFIT);
      }
   }
   
   // Update performance tracking
   g_DailyProfit += totalProfit;
   g_WeeklyProfit += totalProfit;
   g_MonthlyProfit += totalProfit;
   
   // Update panel label
   string profitText = 
      "Daily Profit: $" + DoubleToString(g_DailyProfit, 2) + "\n" +
      "Weekly Profit: $" + DoubleToString(g_WeeklyProfit, 2) + "\n" +
      "Monthly Profit: $" + DoubleToString(g_MonthlyProfit, 2) + "\n" +
      "Total Profit: $" + DoubleToString(totalProfit, 2);
   
   ObjectSetString(0, "ProfitLabel", OBJPROP_TEXT, profitText);
}

//+------------------------------------------------------------------+
//| Check if current time is within valid trading sessions            |
//+------------------------------------------------------------------+
bool IsValidTradingSession()
{
   MqlDateTime dt;
   TimeCurrent(dt);
   int gmtHour = dt.hour;
   
   bool inAsianSession = EnableAsianSession && 
      (gmtHour >= AsianSessionStart && gmtHour < AsianSessionEnd);
   
   bool inLondonSession = EnableLondonSession && 
      (gmtHour >= LondonSessionStart && gmtHour < LondonSessionEnd);
   
   bool inNYOpen = EnableNYOpen && 
      (gmtHour >= NYOpenStart && gmtHour < NYOpenEnd);
   
   bool inNYClose = EnableNYClose && 
      (gmtHour >= NYCloseStart && gmtHour < NYCloseEnd);
   
   return (inAsianSession || inLondonSession || inNYOpen || inNYClose);
}

//+------------------------------------------------------------------+
//| Check if symbol is tradable                                       |
//+------------------------------------------------------------------+
bool IsSymbolTradable(string symbol)
{
   for(int i = 0; i < ArraySize(g_SymbolList); i++)
   {
   


//--- Global objects
CTrade Trade;

//--- Track last signal time to avoid repeated trades
datetime lastSignalTime = 0;

//--- Structure for MqlRates array
MqlRates htfRates[];

//--- Function prototypes
bool IsDoubleTop(int index, MqlRates &rates[], double threshold);
bool IsDoubleBottom(int index, MqlRates &rates[], double threshold);
bool IsHeadAndShoulders(int index, MqlRates &rates[]);
bool IsBullishEngulfing(int index, MqlRates &rates[]);
bool IsPiercingPattern(int index, MqlRates &rates[]);
bool IsMorningStar(int index, MqlRates &rates[]);
bool IsThreeInsideUp(int index, MqlRates &rates[]);
bool IsThreeOutsideUp(int index, MqlRates &rates[]);
bool IsThreeStarInNorth(int index, MqlRates &rates[]);
bool IsHangingMan(int index, MqlRates &rates[]);
bool IsHammer(int index, MqlRates &rates[]);
double CalculateLotSize(double riskPercentage, double stopLossPips);
double CalculateATRStopLoss(int index, MqlRates &rates[], double atrMultiplier);
void ManagePartialProfits(double orderPrice, int orderTicket);
void ApplyTrailingStop(int orderTicket);

//+------------------------------------------------------------------+
//| Check for Double Top Pattern                                      |
//+------------------------------------------------------------------+
bool IsDoubleTop(int index, MqlRates &rates[], double threshold)
{
    double firstPeak = rates[index].high;
    double secondPeak = rates[index + 1].high;
    if(MathAbs(firstPeak - secondPeak) <= threshold)
    {
        return true;
    }
    return false;
}

//+------------------------------------------------------------------+
//| Check for Double Bottom Pattern                                   |
//+------------------------------------------------------------------+
bool IsDoubleBottom(int index, MqlRates &rates[], double threshold)
{
    double firstTrough = rates[index].low;
    double secondTrough = rates[index + 1].low;
    if(MathAbs(firstTrough - secondTrough) <= threshold)
    {
        return true;
    }
    return false;
}

//+------------------------------------------------------------------+
//| Check for Head and Shoulders Pattern                              |
//+------------------------------------------------------------------+
bool IsHeadAndShoulders(int index, MqlRates &rates[])
{
    double leftShoulder = rates[index].high;
    double head = rates[index + 1].high;
    double rightShoulder = rates[index + 2].high;

    return (head > leftShoulder && head > rightShoulder && leftShoulder > rightShoulder);
}

//+------------------------------------------------------------------+
//| Check for Bullish Engulfing Pattern                               |
//+------------------------------------------------------------------+
bool IsBullishEngulfing(int index, MqlRates &rates[])
{
    return (rates[index - 1].close < rates[index].open && rates[index - 1].open > rates[index].close);
}

//+------------------------------------------------------------------+
//| Check for Piercing Pattern                                        |
//+------------------------------------------------------------------+
bool IsPiercingPattern(int index, MqlRates &rates[])
{
    return (rates[index - 1].close < rates[index].open && rates[index].close > rates[index - 1].open);
}

//+------------------------------------------------------------------+
//| Check for Morning Star Pattern                                     |
//+------------------------------------------------------------------+
bool IsMorningStar(int index, MqlRates &rates[])
{
    return (rates[index - 2].close < rates[index - 2].open && rates[index - 1].close < rates[index - 1].open && rates[index].close > rates[index].open);
}

//+------------------------------------------------------------------+
//| Check for Three Inside Up Pattern                                  |
//+------------------------------------------------------------------+
bool IsThreeInsideUp(int index, MqlRates &rates[])
{
    return (rates[index - 2].close < rates[index - 2].open && rates[index - 1].close > rates[index - 1].open && rates[index].close > rates[index - 1].close);
}

//+------------------------------------------------------------------+
//| OnInit                                                           |
//+------------------------------------------------------------------+
int OnInit()
{
   EventSetMillisecondTimer(RefreshTimer);
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| OnDeinit                                                         |
//+------------------------------------------------------------------+
int OnDeinit()
{
   EventKillTimer();
   Comment("");
   return(0);
}

//+------------------------------------------------------------------+
//| OnTimer                                                          |
//+------------------------------------------------------------------+
void OnTimer()
{
   OnTick();
}

//+------------------------------------------------------------------+
//| OnTick                                                           |
//+------------------------------------------------------------------+
void OnTick()
{
   static datetime lastBarTime = 0;
   datetime curBarTime = iTime(Symbol(), PERIOD_CURRENT, 0);
   if(curBarTime == lastBarTime) return;
   lastBarTime = curBarTime;

   ArraySetAsSeries(htfRates, true);
   int copied = CopyRates(Symbol(), HTF, 0, LookbackBars, htfRates);
   if(copied <= 0)
   {
      if(Show_Debug_Comments) Comment("No HTF rates copied.");
      return;
   }

   int lastPatternIndex = -1;
   bool foundDT = false;
   bool foundDB = false;

   for(int i = 2; i < copied - 3; i++)
   {
      if(IsDoubleTop(i, htfRates, DT_Threshold))
      {
         foundDT = true;
         lastPatternIndex = i;
         break;
      }
      if(IsDoubleBottom(i, htfRates, DT_Threshold))
      {
         foundDB = true;
         lastPatternIndex = i;
         break;
      }
   }

   if(lastPatternIndex < 0)
   {
      if(Show_Debug_Comments) Comment("No DT/DB found in the last ", LookbackBars, " bars on ", EnumToString(HTF));
      return;
   }

   bool bullishEngulfing = IsBullishEngulfing(lastPatternIndex, htfRates);
   bool piercing = IsPiercingPattern(lastPatternIndex, htfRates);
   bool morningStar = IsMorningStar(lastPatternIndex, htfRates);
   bool threeInsideUp = IsThreeInsideUp(lastPatternIndex, htfRates);

   string msg;
   if(foundDT) msg += "Double Top found. ";
   if(foundDB) msg += "Double Bottom found. ";
   if(bullishEngulfing) msg += "Bullish Engulfing. ";
   if(piercing) msg += "Piercing Pattern. ";
   if(morningStar) msg += "Morning Star. ";
   if(threeInsideUp) msg += "3 Inside Up. ";

   if(Show_Debug_Comments) Comment(msg);

   if(foundDB && (bullishEngulfing || piercing || morningStar || threeInsideUp))
   {
      datetime signalTime = htfRates[lastPatternIndex].time;
      if(OneTradePerSignal && signalTime == lastSignalTime) return;
      
      if(Enable_Buy)
      {
         double point = SymbolInfoDouble(Symbol(), SYMBOL_POINT);
         double price = SymbolInfoDouble(Symbol(), SYMBOL_ASK);

         double sl = UseATRStopLoss ? CalculateATRStopLoss(lastPatternIndex, htfRates, ATROffset) : price - StopLossPips * point;
         double tp = price + TakeProfitPips * point;
         double lotSize = CalculateLotSize(RiskPercentage, StopLossPips);

         Trade.SetExpertMagicNumber(MagicNumber);
         bool result = Trade.Buy(lotSize, NULL, price, sl, tp, "DB+BullishPattern");
         if(result)
         {
            int orderTicket = Trade.ResultOrder();
            Print("Buy trade opened at ", price, " due to DB + bullish pattern.");
            lastSignalTime = signalTime;

            ManagePartialProfits(price, orderTicket);
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Calculate Lot Size as Percentage of Equity                       |
//+------------------------------------------------------------------+
double CalculateLotSize(double riskPercentage, double stopLossPips)
{
   double equity = AccountInfoDouble(ACCOUNT_EQUITY);
   double riskAmount = equity * (riskPercentage / 100.0);
   double tickValue = SymbolInfoDouble(Symbol(), SYMBOL_TRADE_TICK_VALUE);
   double lotSize = riskAmount / (stopLossPips * tickValue);
   double minLot = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MIN);
   double maxLot = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MAX);
   double stepLot = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_STEP);

   // Ensure the lot size is within allowed limits and conforms to the step size
   lotSize = MathMax(minLot, MathMin(lotSize, maxLot));
   return(NormalizeDouble(lotSize / stepLot, 0) * stepLot);
}

//+------------------------------------------------------------------+
//| Calculate ATR-Based Stop Loss                                    |
//+------------------------------------------------------------------+
double CalculateATRStopLoss(int index, MqlRates &rates[], double atrMultiplier)
{
   int shift = ArraySize(rates) - index - 1;

   // Create an ATR handle
   int atrHandle = iATR(_Symbol, HTF, 14);
   if(atrHandle == INVALID_HANDLE)
   {
      Print("Failed to create ATR handle");
      return 0.0;
   }

   double atrBuffer[];
   if(CopyBuffer(atrHandle,0,shift,1,atrBuffer)<=0)
   {
      Print("Failed to copy ATR data");
      return 0.0;
   }

   double atr = atrBuffer[0];
   return (rates[index].low - atr * atrMultiplier);
}

//+------------------------------------------------------------------+
//| Manage Partial Profits                                           |
//+------------------------------------------------------------------+
void ManagePartialProfits(double orderPrice, int orderTicket)
{
    double point = SymbolInfoDouble(Symbol(), SYMBOL_POINT);
    double profitLevels[] = {
        orderPrice + PartialProfit1Percent/100 * point,
        orderPrice + PartialProfit2Percent/100 * point,
        orderPrice + PartialProfit3Percent/100 * point,
        orderPrice + PartialProfit4Percent/100 * point,
        orderPrice + PartialProfit5Percent/100 * point
    };

    double percentages[] = {
        PartialProfit1Percent,
        PartialProfit2Percent,
        PartialProfit3Percent,
        PartialProfit4Percent,
        PartialProfit5Percent
    };

    for (int i = 0; i < ArraySize(profitLevels); i++)
    {
        if (profitLevels[i] == DBL_MAX) continue; // Skip already closed levels

        double currentPrice = SymbolInfoDouble(Symbol(), SYMBOL_BID);
        if (currentPrice >= profitLevels[i])
        {
            if (Trade.PositionClosePartial(orderTicket, percentages[i] / 100.0))
            {
                Print("Partial profit closed at level ", i + 1);
                profitLevels[i] = DBL_MAX; // Mark as processed
            }
        }
    }
}

//+------------------------------------------------------------------+
//| Apply Trailing Stop                                              |
//+------------------------------------------------------------------+
void ApplyTrailingStop(int orderTicket)
{
   double point = SymbolInfoDouble(Symbol(), SYMBOL_POINT);
   double currentPrice = SymbolInfoDouble(Symbol(), SYMBOL_BID);

   if(!PositionSelectByTicket(orderTicket))
   {
      Print("Error: Unable to select position with ticket ", orderTicket);
      return;
   }

   double currentSL = PositionGetDouble(POSITION_SL);
   if(currentSL == 0.0 || currentSL < currentPrice - TrailingStopPips * point)
   {
      double newSL = currentPrice - TrailingStopPips * point;
      Trade.PositionModify(orderTicket, newSL, 0);
      Print("Trailing Stop updated to: ", newSL);
   }
}
